<?php

namespace Teknisa\PLA\Util;

use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Error;
use PhpOffice\PhpSpreadsheet\Cell\Coordinate;
use PhpOffice\PhpSpreadsheet\Worksheet;

class TableUtil{ 
    
    public function __construct() {}
    
    //Cria a associação entre nomes e index de colunas na planilha
    protected static function buildColumnNameReference($worksheet, $maxColumn, $columnNames){
        $reference = array();
        $column = 1;

            for($column; $column <= $maxColumn; $column++){
                $columnString = self::getColumnString($column);
                $columnName = $worksheet->getCell($columnString.'1')->getValue();

                $reference[$columnString] = $column;
           
             }
        return ($reference);
    }
    
    protected static function getColumnIndex($column){
        if(gettype($column) == 'string'){
            return Coordinate::columnIndexFromString($column);
        }
        else
            return $column;
    }
    
    protected static function getColumnString($column){
        if(gettype($column) == 'integer'){
            return Coordinate::stringFromColumnIndex($column);
        }
        else
            return $column;
    }
    
    
    /* Lê uma tabela na planilha e retorna um objeto Table com as informações
    *  referentes.
    */
    public static function readWorksheetTable($worksheet, $columnNames, $minRowIndex = 2/*, $maxRowIndex = NULL*/){
    
        //configura as dimensões da tabela
        $worksheetDimensions = $worksheet->getHighestRowAndColumn();
        // if($maxRowIndex === NULL){
        //     $maxRowIndex = $worksheetDimensions['row'];
        // }
        $maxRowIndex = $worksheetDimensions['row'];
        
        $maxColumnIndex = self::getColumnIndex($worksheetDimensions['column']);

        $columnsReference = self::buildColumnNameReference($worksheet, $maxColumnIndex, $columnNames);
        /*Lê o arquivo construindo a linha e monta um array de arrays associativos
        * (linhas da tabela) e cada valor da linha é acessado por uma chave que
        * é o nome da coluna na planilha
        */

        $tableRows = array();
        for($row = $minRowIndex; $row <= $maxRowIndex; $row++){
            $worksheetRowContent = array();
            $rowIsEmpty = true;
            
            $column = 1;
            
            for($column; $column <= $maxColumnIndex; $column++){
                $columnLetterIndex = self::getColumnString($column);
                $columnName = $columnsReference[$columnLetterIndex];
                $cellValue = $worksheet->getCell($columnLetterIndex.$row)->getValue();
                if($cellValue !== NULL){
                    $rowIsEmpty = false;
                    if(strstr($cellValue,'=')==true){
                        $cellValue = $worksheet->getCell($columnLetterIndex.$row)->getFormattedValue();
                    }
                }
                $worksheetRowContent = array_merge($worksheetRowContent, array($columnName => $cellValue));
            }
            if(!$rowIsEmpty){
                $worksheetRow = array(
                    'index' => $row,
                    'content' => $worksheetRowContent
                );
                array_push($tableRows, $worksheetRow);
            }
        }
        
        /* Retorna o dados da tabela: um array de referencia que retorna o 
        * titulo da coluna pra cada chave com o valor de um index da planilha e 
        * as linhas da tabela.
        */
        $table = new Table($columnsReference, $tableRows);
        return $table;
    }
    
}